@@ -310,8 +310,6 @@ struct SplM_field_ext_info: public SplM_field_info
        occurred also in the select list of this join
     9. There are defined some keys usable for ref access of fields from C
        with available statistics.
-    10. The select doesn't use WITH ROLLUP (This limitation can probably be
-       lifted)
 
   @retval
     true   if the answer is positive
@@ -328,8 +326,7 @@ bool JOIN::check_for_splittable_materialized()
       (unit->first_select()->next_select()) ||                        // !(3)
       (derived->prohibit_cond_pushdown) ||                            // !(4)
       (derived->is_recursive_with_table()) ||                         // !(5)
-      (table_count == 0 || const_tables == top_join_tab_count) ||     // !(6)
-      rollup.state != ROLLUP::STATE_NONE)                             // (10)
+      (table_count == 0 || const_tables == top_join_tab_count))       // !(6)
     return false;
   if (group_list)                                                     // (7.1)
   {
@@ -346,7 +343,7 @@ bool JOIN::check_for_splittable_materialized()
     return false;
 
   ORDER *ord;
-  Dynamic_array<SplM_field_ext_info> candidates;
+  Dynamic_array<SplM_field_ext_info> candidates(PSI_INSTRUMENT_MEM);
 
   /*
     Select from partition_list all candidates for splitting.
@@ -717,7 +714,7 @@ void JOIN::add_keyuses_for_splitting()
   KEY_FIELD *added_key_field;
   if (!spl_opt_info->added_key_fields.elements)
     goto err;
-  if (!(ext_keyuses_for_splitting= new Dynamic_array<KEYUSE_EXT>))
+  if (!(ext_keyuses_for_splitting= new Dynamic_array<KEYUSE_EXT>(PSI_INSTRUMENT_MEM)))
     goto err;
   while ((added_key_field= li++))
   {
@@ -747,13 +744,11 @@ void JOIN::add_keyuses_for_splitting()
   save_query_plan(save_qep);
 
   if (!keyuse.buffer &&
-       my_init_dynamic_array(&keyuse, sizeof(KEYUSE), 20, 64,
-                             MYF(MY_THREAD_SPECIFIC)))
+       my_init_dynamic_array(PSI_INSTRUMENT_ME, &keyuse, sizeof(KEYUSE),
+                             20, 64, MYF(MY_THREAD_SPECIFIC)))
     goto err;
 
-  if (allocate_dynamic(&keyuse,
-                       save_qep->keyuse.elements +
-                       added_keyuse_count))
+  if (allocate_dynamic(&keyuse, save_qep->keyuse.elements + added_keyuse_count))
     goto err;
 
   idx= keyuse.elements= save_qep->keyuse.elements;
@@ -963,7 +958,11 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
       in the cache
     */
     spl_plan= spl_opt_info->find_plan(best_table, best_key, best_key_parts);
-    if (!spl_plan)
+    if (!spl_plan &&
+	(spl_plan= (SplM_plan_info *) thd->alloc(sizeof(SplM_plan_info))) &&
+	(spl_plan->best_positions=
+	   (POSITION *) thd->alloc(sizeof(POSITION) * join->table_count)) &&
+	!spl_opt_info->plan_cache.push_back(spl_plan))
     {
       /*
         The plan for the chosen key has not been found in the cache.
@@ -973,27 +972,6 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
       reset_validity_vars_for_keyuses(best_key_keyuse_ext_start, best_table,
                                       best_key, remaining_tables, true);
       choose_plan(join, all_table_map & ~join->const_table_map);
-
-      /*
-        Check that the chosen plan is really a splitting plan.
-        If not or if there is not enough memory to save the plan in the cache
-        then just return with no splitting plan.
-      */
-      POSITION *first_non_const_pos= join->best_positions + join->const_tables;
-      TABLE *table= first_non_const_pos->table->table;
-      key_map spl_keys= table->keys_usable_for_splitting;
-      if (!(first_non_const_pos->key &&
-            spl_keys.is_set(first_non_const_pos->key->key)) ||
-          !(spl_plan= (SplM_plan_info *) thd->alloc(sizeof(SplM_plan_info))) ||
-	  !(spl_plan->best_positions=
-	     (POSITION *) thd->alloc(sizeof(POSITION) * join->table_count)) ||
-	  spl_opt_info->plan_cache.push_back(spl_plan))
-      {
-        reset_validity_vars_for_keyuses(best_key_keyuse_ext_start, best_table,
-                                        best_key, remaining_tables, false);
-        return 0;
-      }
-
       spl_plan->keyuse_ext_start= best_key_keyuse_ext_start;
       spl_plan->table= best_table;
       spl_plan->key= best_key;
@@ -1018,7 +996,7 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
     }
     if (spl_plan)
     {
-      if(record_count * spl_plan->cost < spl_opt_info->unsplit_cost)
+      if(record_count * spl_plan->cost < spl_opt_info->unsplit_cost - 0.01)
       {
         /*
           The best plan that employs splitting is cheaper than
@@ -1048,16 +1026,16 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
     Inject equalities for splitting used by the materialization join
 
   @param
-    excluded_tables  used to filter out the equalities that cannot
+    remaining_tables  used to filter out the equalities that cannot
                       be pushed.
 
   @details
-    This function injects equalities pushed into a derived table T for which
-    the split optimization has been chosen by the optimizer. The function
-    is called by JOIN::inject_splitting_cond_for_all_tables_with_split_op().
-    All equalities usable for splitting T whose right parts do not depend on
-    any of the 'excluded_tables' can be pushed into the where clause of the
-    derived table T.
+    This function is called by JOIN_TAB::fix_splitting that is used
+    to fix the chosen splitting of a splittable materialized table T
+    in the final query execution plan. In this plan the table T
+    is joined just before the 'remaining_tables'. So all equalities
+    usable for splitting whose right parts do not depend on any of
+    remaining tables can be pushed into join for T.
     The function also marks the select that specifies T as
     UNCACHEABLE_DEPENDENT_INJECTED.
 
@@ -1066,7 +1044,7 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(double record_count,
     true   on failure
 */
 
-bool JOIN::inject_best_splitting_cond(table_map excluded_tables)
+bool JOIN::inject_best_splitting_cond(table_map remaining_tables)
 {
   Item *inj_cond= 0;
   List<Item> *inj_cond_list= &spl_opt_info->inj_cond_list;
@@ -1074,7 +1052,7 @@ bool JOIN::inject_best_splitting_cond(table_map excluded_tables)
   KEY_FIELD *added_key_field;
   while ((added_key_field= li++))
   {
-    if (excluded_tables & added_key_field->val->used_tables())
+    if (remaining_tables & added_key_field->val->used_tables())
       continue;
     if (inj_cond_list->push_back(added_key_field->cond, thd->mem_root))
       return true;
@@ -1091,7 +1069,7 @@ bool JOIN::inject_best_splitting_cond(table_map excluded_tables)
   if (inj_cond)
     inj_cond->fix_fields(thd,0);
 
-  if (inject_cond_into_where(inj_cond->copy_andor_structure(thd)))
+  if (inject_cond_into_where(inj_cond))
     return true;
 
   select_lex->uncacheable|= UNCACHEABLE_DEPENDENT_INJECTED;
@@ -1168,6 +1146,8 @@ bool JOIN_TAB::fix_splitting(SplM_plan_info *spl_plan,
     memcpy((char *) md_join->best_positions,
            (char *) spl_plan->best_positions,
            sizeof(POSITION) * md_join->table_count);
+    if (md_join->inject_best_splitting_cond(remaining_tables))
+      return true;
     /*
       This is called for a proper work of JOIN::get_best_combination()
       called for the join that materializes T
@@ -1211,8 +1191,7 @@ bool JOIN::fix_all_splittings_in_plan()
     if (tab->table->is_splittable())
     {
       SplM_plan_info *spl_plan= cur_pos->spl_plan;
-      if (tab->fix_splitting(spl_plan,
-                             all_tables & ~prev_tables,
+      if (tab->fix_splitting(spl_plan, all_tables & ~prev_tables,
                              tablenr < const_tables ))
           return true;
     }
@@ -1220,44 +1199,3 @@ bool JOIN::fix_all_splittings_in_plan()
   }
   return false;
 }
-
-
-/**
-  @brief
-    Inject splitting conditions into WHERE of split derived
-
-  @details
-    The function calls JOIN_TAB::inject_best_splitting_cond() for each
-    materialized derived table T used in this join for which the split
-    optimization has been chosen by the optimizer. It is done in order to
-    inject equalities pushed into the where clause of the specification
-    of T that would be helpful to employ the splitting technique.
-
-  @retval
-    false  on success
-    true   on failure
-*/
-
-bool JOIN::inject_splitting_cond_for_all_tables_with_split_opt()
-{
-  table_map prev_tables= 0;
-  table_map all_tables= (table_map(1) << table_count) - 1;
-  for (uint tablenr= 0; tablenr < table_count; tablenr++)
-  {
-    POSITION *cur_pos= &best_positions[tablenr];
-    JOIN_TAB *tab= cur_pos->table;
-    prev_tables|= tab->table->map;
-    if (!(tab->table->is_splittable() && cur_pos->spl_plan))
-      continue;
-    SplM_opt_info *spl_opt_info= tab->table->spl_opt_info;
-    JOIN *join= spl_opt_info->join;
-    /*
-      Currently the equalities referencing columns of SJM tables with
-      look-up access cannot be pushed into materialized derived.
-    */
-    if (join->inject_best_splitting_cond((all_tables & ~prev_tables) |
-				          sjm_lookup_tables))
-        return true;
-  }
-  return false;
-}
