@@ -29,7 +29,7 @@
 
 #define MYSQL_LEX 1
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "procedure.h"
 #include "sql_analyse.h"
@@ -68,11 +68,30 @@ int compare_decimal2(int* len, const char *s, const char *t)
 }
 
 
+static bool
+prepare_param(THD *thd, Item **item, const char *proc_name, uint pos)
+{
+  if ((*item)->fix_fields_if_needed(thd, item))
+  {
+    DBUG_PRINT("info", ("fix_fields() for the parameter %u failed", pos));
+    return true;
+  }
+  if ((*item)->type_handler()->result_type() != INT_RESULT ||
+      !(*item)->basic_const_item() ||
+      (*item)->val_real() < 0)
+  {
+    my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
+    return true;
+  }
+  return false;
+}
+
+
 Procedure *
 proc_analyse_init(THD *thd, ORDER *param, select_result *result,
 		  List<Item> &field_list)
 {
-  char *proc_name = (*param->item)->name;
+  const char *proc_name = (*param->item)->name.str;
   analyse *pc = new analyse(result);
   field_info **f_info;
   DBUG_ENTER("proc_analyse_init");
@@ -88,17 +107,8 @@ proc_analyse_init(THD *thd, ORDER *param, select_result *result,
   else if (param->next)
   {
     // first parameter
-    if (!(*param->item)->fixed && (*param->item)->fix_fields(thd, param->item))
-    {
-      DBUG_PRINT("info", ("fix_fields() for the first parameter failed"));
+    if (prepare_param(thd, param->item, proc_name, 0))
       goto err;
-    }
-    if ((*param->item)->type() != Item::INT_ITEM ||
-	(*param->item)->val_real() < 0)
-    {
-      my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
-      goto err;
-    }
     pc->max_tree_elements = (uint) (*param->item)->val_int();
     param = param->next;
     if (param->next)  // no third parameter possible
@@ -107,25 +117,12 @@ proc_analyse_init(THD *thd, ORDER *param, select_result *result,
       goto err;
     }
     // second parameter
-    if (!(*param->item)->fixed && (*param->item)->fix_fields(thd, param->item))
-    {
-      DBUG_PRINT("info", ("fix_fields() for the second parameter failed"));
+    if (prepare_param(thd, param->item, proc_name, 1))
       goto err;
-    }
-    if ((*param->item)->type() != Item::INT_ITEM ||
-	(*param->item)->val_real() < 0)
-    {
-      my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
-      goto err;
-    }
     pc->max_treemem = (uint) (*param->item)->val_int();
   }
-  else if ((*param->item)->type() != Item::INT_ITEM ||
-	   (*param->item)->val_real() < 0)
-  {
-    my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
+  else if (prepare_param(thd, param->item, proc_name, 0))
     goto err;
-  }
   // if only one parameter was given, it will be the value of max_tree_elements
   else
   {
@@ -298,9 +295,10 @@ bool get_ev_num_info(EV_NUM_INFO *ev_info, NUM_INFO *info, const char *num)
 } // get_ev_num_info
 
 
-void free_string(void* str, TREE_FREE, void*)
+int free_string(void* str, TREE_FREE, void*)
 {
   ((String*)str)->free();
+  return 0;
 }
 
 
@@ -374,7 +372,7 @@ void field_str::add()
       if (!tree_insert(&tree, (void*) &s, 0, tree.custom_arg))
       {
 	room_in_tree = 0;      // Remove tree, out of RAM ?
-	delete_tree(&tree);
+	delete_tree(&tree, 0);
       }
       else
       {
@@ -382,7 +380,7 @@ void field_str::add()
 	if ((treemem += length) > pc->max_treemem)
 	{
 	  room_in_tree = 0;	 // Remove tree, too big tree
-	  delete_tree(&tree);
+	  delete_tree(&tree, 0);
 	}
       }
     }
@@ -441,7 +439,7 @@ void field_real::add()
     if (!(element = tree_insert(&tree, (void*) &num, 0, tree.custom_arg)))
     {
       room_in_tree = 0;    // Remove tree, out of RAM ?
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
     /*
       if element->count == 1, this element can be found only once from tree
@@ -450,7 +448,7 @@ void field_real::add()
     else if (element->count == 1 && (tree_elements++) >= pc->max_tree_elements)
     {
       room_in_tree = 0;  // Remove tree, too many elements
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
   }
 
@@ -480,34 +478,32 @@ void field_real::add()
 void field_decimal::add()
 {
   /*TODO - remove rounding stuff after decimal_div returns proper frac */
-  my_decimal dec_buf, *dec= item->val_decimal(&dec_buf);
-  my_decimal rounded;
+  VDec vdec(item);
   uint length;
   TREE_ELEMENT *element;
 
-  if (item->null_value)
+  if (vdec.is_null())
   {
     nulls++;
     return;
   }
 
-  my_decimal_round(E_DEC_FATAL_ERROR, dec, item->decimals, FALSE,&rounded);
-  dec= &rounded;
+  my_decimal dec;
+  vdec.round_to(&dec, item->decimals, HALF_UP);
 
-  length= my_decimal_string_length(dec);
+  length= my_decimal_string_length(&dec);
 
-  if (decimal_is_zero(dec))
+  if (decimal_is_zero(&dec))
     empty++;
 
   if (room_in_tree)
   {
     uchar buf[DECIMAL_MAX_FIELD_SIZE];
-    my_decimal2binary(E_DEC_FATAL_ERROR, dec, buf,
-                      item->max_length, item->decimals);
+    dec.to_binary(buf, item->max_length, item->decimals);
     if (!(element = tree_insert(&tree, (void*)buf, 0, tree.custom_arg)))
     {
       room_in_tree = 0;    // Remove tree, out of RAM ?
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
     /*
       if element->count == 1, this element can be found only once from tree
@@ -516,25 +512,25 @@ void field_decimal::add()
     else if (element->count == 1 && (tree_elements++) >= pc->max_tree_elements)
     {
       room_in_tree = 0;  // Remove tree, too many elements
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
   }
 
   if (!found)
   {
     found = 1;
-    min_arg = max_arg = sum[0] = *dec;
-    my_decimal_mul(E_DEC_FATAL_ERROR, sum_sqr, dec, dec);
+    min_arg = max_arg = sum[0] = dec;
+    my_decimal_mul(E_DEC_FATAL_ERROR, sum_sqr, &dec, &dec);
     cur_sum= 0;
     min_length = max_length = length;
   }
-  else if (!decimal_is_zero(dec))
+  else if (!decimal_is_zero(&dec))
   {
     int next_cur_sum= cur_sum ^ 1;
     my_decimal sqr_buf;
 
-    my_decimal_add(E_DEC_FATAL_ERROR, sum+next_cur_sum, sum+cur_sum, dec);
-    my_decimal_mul(E_DEC_FATAL_ERROR, &sqr_buf, dec, dec);
+    my_decimal_add(E_DEC_FATAL_ERROR, sum+next_cur_sum, sum+cur_sum, &dec);
+    my_decimal_mul(E_DEC_FATAL_ERROR, &sqr_buf, &dec, &dec);
     my_decimal_add(E_DEC_FATAL_ERROR,
                    sum_sqr+next_cur_sum, sum_sqr+cur_sum, &sqr_buf);
     cur_sum= next_cur_sum;
@@ -542,13 +538,13 @@ void field_decimal::add()
       min_length = length;
     if (length > max_length)
       max_length = length;
-    if (my_decimal_cmp(dec, &min_arg) < 0)
+    if (dec.cmp(&min_arg) < 0)
     {
-      min_arg= *dec;
+      min_arg= dec;
     }
-    if (my_decimal_cmp(dec, &max_arg) > 0)
+    if (dec.cmp(&max_arg) > 0)
     {
-      max_arg= *dec;
+      max_arg= dec;
     }
   }
 }
@@ -574,7 +570,7 @@ void field_longlong::add()
     if (!(element = tree_insert(&tree, (void*) &num, 0, tree.custom_arg)))
     {
       room_in_tree = 0;    // Remove tree, out of RAM ?
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
     /*
       if element->count == 1, this element can be found only once from tree
@@ -583,7 +579,7 @@ void field_longlong::add()
     else if (element->count == 1 && (tree_elements++) >= pc->max_tree_elements)
     {
       room_in_tree = 0;  // Remove tree, too many elements
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
   }
 
@@ -630,7 +626,7 @@ void field_ulonglong::add()
     if (!(element = tree_insert(&tree, (void*) &num, 0, tree.custom_arg)))
     {
       room_in_tree = 0;    // Remove tree, out of RAM ?
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
     /*
       if element->count == 1, this element can be found only once from tree
@@ -639,7 +635,7 @@ void field_ulonglong::add()
     else if (element->count == 1 && (tree_elements++) >= pc->max_tree_elements)
     {
       room_in_tree = 0;  // Remove tree, too many elements
-      delete_tree(&tree);
+      delete_tree(&tree, 0);
     }
   }
 
@@ -1002,7 +998,7 @@ void field_decimal::get_opt_type(String *answer,
   uint length;
 
   my_decimal_set_zero(&zero);
-  my_bool is_unsigned= (my_decimal_cmp(&zero, &min_arg) >= 0);
+  my_bool is_unsigned= (zero.cmp(&min_arg) >= 0);
 
   length= sprintf(buff, "DECIMAL(%d, %d)",
                   (int) (max_length - (item->decimals ? 1 : 0)),
@@ -1015,14 +1011,14 @@ void field_decimal::get_opt_type(String *answer,
 
 String *field_decimal::get_min_arg(String *str)
 {
-  my_decimal2string(E_DEC_FATAL_ERROR, &min_arg, 0, 0, '0', str);
+  min_arg.to_string_native(str, 0, 0, '0');
   return str;
 }
 
 
 String *field_decimal::get_max_arg(String *str)
 {
-  my_decimal2string(E_DEC_FATAL_ERROR, &max_arg, 0, 0, '0', str);
+  max_arg.to_string_native(str, 0, 0, '0');
   return str;
 }
 
@@ -1040,10 +1036,10 @@ String *field_decimal::avg(String *s, ha_rows rows)
   int2my_decimal(E_DEC_FATAL_ERROR, rows - nulls, FALSE, &num);
   my_decimal_div(E_DEC_FATAL_ERROR, &avg_val, sum+cur_sum, &num, prec_increment);
   /* TODO remove this after decimal_div returns proper frac */
-  my_decimal_round(E_DEC_FATAL_ERROR, &avg_val,
+  avg_val.round_to(&rounded_avg,
                    MY_MIN(sum[cur_sum].frac + prec_increment, DECIMAL_MAX_SCALE),
-                   FALSE,&rounded_avg);
-  my_decimal2string(E_DEC_FATAL_ERROR, &rounded_avg, 0, 0, '0', s);
+                   HALF_UP);
+  rounded_avg.to_string_native(s, 0, 0, '0');
   return s;
 }
 
@@ -1056,7 +1052,6 @@ String *field_decimal::std(String *s, ha_rows rows)
     return s;
   }
   my_decimal num, tmp, sum2, sum2d;
-  double std_sqr;
   int prec_increment= current_thd->variables.div_precincrement;
 
   int2my_decimal(E_DEC_FATAL_ERROR, rows - nulls, FALSE, &num);
@@ -1064,7 +1059,7 @@ String *field_decimal::std(String *s, ha_rows rows)
   my_decimal_div(E_DEC_FATAL_ERROR, &tmp, &sum2, &num, prec_increment);
   my_decimal_sub(E_DEC_FATAL_ERROR, &sum2, sum_sqr+cur_sum, &tmp);
   my_decimal_div(E_DEC_FATAL_ERROR, &tmp, &sum2, &num, prec_increment);
-  my_decimal2double(E_DEC_FATAL_ERROR, &tmp, &std_sqr);
+  double std_sqr= tmp.to_double();
   s->set_real(((double) std_sqr <= 0.0 ? 0.0 : sqrt(std_sqr)),
          MY_MIN(item->decimals + prec_increment, NOT_FIXED_DEC), my_thd_charset);
 
@@ -1116,12 +1111,9 @@ int collect_decimal(uchar *element, element_count count,
     info->str->append(',');
   else
     info->found = 1;
-  my_decimal dec;
-  binary2my_decimal(E_DEC_FATAL_ERROR, element, &dec,
-                    info->item->max_length, info->item->decimals);
-  
+  my_decimal dec(element, info->item->max_length, info->item->decimals);
   info->str->append('\'');
-  my_decimal2string(E_DEC_FATAL_ERROR, &dec, 0, 0, '0', &s);
+  dec.to_string_native(&s, 0, 0, '0');
   info->str->append(s);
   info->str->append('\'');
   return 0;
@@ -1241,4 +1233,3 @@ uint check_ulonglong(const char *str, uint length)
   while (*cmp && *cmp++ == *str++) ;
   return ((uchar) str[-1] <= (uchar) cmp[-1]) ? smaller : bigger;
 } /* check_ulonlong */
-
