@@ -35,11 +35,11 @@
 #include "my_sys.h"
 #include "mysqld_error.h"
 #endif   // !MYSQL_PREPARED_STATEMENTS
-#if defined(_WIN32)
+#if defined(__WIN__)
 //#include <windows.h>
-#else   // !_WIN32
+#else   // !__WIN__
 #include "osutil.h"
-#endif  // !_WIN32
+#endif  // !__WIN__
 
 #include "global.h"
 #include "plgdbsem.h"
@@ -88,8 +88,8 @@ static MYSQL_RES *connect_use_result(MYSQL *mysql)
     DBUG_RETURN(NULL);
     } // endif status
 
-  if (!(result = (MYSQL_RES*) my_malloc(sizeof(*result) +
-				          sizeof(ulong) * mysql->field_count,
+  if (!(result = (MYSQL_RES*) my_malloc(PSI_NOT_INSTRUMENTED,
+                  sizeof(*result) + sizeof(ulong) * mysql->field_count,
 				          MYF(MY_WME | MY_ZEROFILL))))
     DBUG_RETURN(NULL);
 
@@ -97,8 +97,8 @@ static MYSQL_RES *connect_use_result(MYSQL *mysql)
   result->methods = mysql->methods;
 
   /* Ptrs: to one row */
-  if (!(result->row = (MYSQL_ROW)my_malloc(sizeof(result->row[0]) *
-                                (mysql->field_count+1), MYF(MY_WME)))) {
+  if (!(result->row = (MYSQL_ROW)my_malloc(PSI_NOT_INSTRUMENTED,
+                sizeof(result->row[0]) * (mysql->field_count+1), MYF(MY_WME)))) {
     my_free(result);
     DBUG_RETURN(NULL);
     }  // endif row
@@ -120,7 +120,7 @@ static MYSQL_RES *connect_use_result(MYSQL *mysql)
 /************************************************************************/
 /*  MyColumns: constructs the result blocks containing all columns      */
 /*  of a MySQL table or view.                                           */
-/*  info = TRUE to get catalog column informations.                     */
+/*  info = TRUE to get catalog column information.                     */
 /************************************************************************/
 PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
                   const char *user, const char *pwd,
@@ -492,15 +492,15 @@ int MYSQLC::Open(PGLOBAL g, const char *host, const char *db,
 //mysql_options(m_DB, MYSQL_OPT_READ_TIMEOUT, &nrt);
 //mysql_options(m_DB, MYSQL_OPT_WRITE_TIMEOUT, ...);
 
-#if defined(_WIN32)
+#if defined(__WIN__)
   if (!strcmp(host, ".")) {
     mysql_options(m_DB, MYSQL_OPT_NAMED_PIPE, NULL);
     pipe = mysqld_unix_port;
     } // endif host
-#else   // !_WIN32
+#else   // !__WIN__
   if (!strcmp(host, "localhost"))
     pipe = mysqld_unix_port;
-#endif  // !_WIN32
+#endif  // !__WIN__
 
 #if 0
   if (pwd && !strcmp(pwd, "*")) {
