@@ -153,7 +153,7 @@ int ha_heap::close(void)
 
 handler *ha_heap::clone(const char *name, MEM_ROOT *mem_root)
 {
-  handler *new_handler= get_new_handler(table->s, mem_root, table->s->db_type());
+  handler *new_handler= get_new_handler(table->s, mem_root, ht);
   if (new_handler && !new_handler->ha_open(table, file->s->name, table->db_stat,
                                            HA_OPEN_IGNORE_IF_LOCKED))
     return new_handler;
@@ -367,9 +367,6 @@ int ha_heap::info(uint flag)
 {
   HEAPINFO hp_info;
 
-  if (!file)
-    return 0;
-
   (void) heap_info(file,&hp_info,flag);
 
   errkey=                     hp_info.errkey;
