@@ -637,14 +637,14 @@ Use MONITOR_DEC if appropriate mutex protection exists.
 		}							\
 	}
 
-#ifdef HAVE_valgrind
+#ifdef HAVE_MEM_CHECK
 # define MONITOR_CHECK_DEFINED(value) do {	\
 	mon_type_t m = value;			\
 	MEM_CHECK_DEFINED(&m, sizeof m);	\
 } while (0)
-#else /* HAVE_valgrind */
+#else /* HAVE_MEM_CHECK */
 # define MONITOR_CHECK_DEFINED(value) (void) 0
-#endif /* HAVE_valgrind */
+#endif /* HAVE_MEM_CHECK */
 
 #define	MONITOR_INC_VALUE(monitor, value)				\
 	MONITOR_CHECK_DEFINED(value);					\
