@@ -215,9 +215,8 @@ static CONTROL_FILE_ERROR create_control_file(const char *name,
   file.
 */
 
-static int lock_control_file(const char *name)
+static int lock_control_file(const char *name, my_bool do_retry)
 {
-  uint retry= 0;
   /*
     On Windows, my_lock() uses locking() which is mandatory locking and so
     prevents maria-recovery.test from copying the control file. And in case of
@@ -228,6 +227,9 @@ static int lock_control_file(const char *name)
     file under Windows.
   */
 #ifndef __WIN__
+  uint retry= 0;
+  uint retry_count= do_retry ? MARIA_MAX_CONTROL_FILE_LOCK_RETRY : 0;
+
   /*
     We can't here use the automatic wait in my_lock() as the alarm thread
     may not yet exists.
@@ -239,8 +241,8 @@ static int lock_control_file(const char *name)
       my_printf_error(HA_ERR_INITIALIZATION,
                       "Can't lock aria control file '%s' for exclusive use, "
                       "error: %d. Will retry for %d seconds", 0,
-                      name, my_errno, MARIA_MAX_CONTROL_FILE_LOCK_RETRY);
-    if (retry++ > MARIA_MAX_CONTROL_FILE_LOCK_RETRY)
+                      name, my_errno, retry_count);
+    if (++retry > retry_count)
       return 1;
     sleep(1);
   }
@@ -269,7 +271,8 @@ static int lock_control_file(const char *name)
 */
 
 CONTROL_FILE_ERROR ma_control_file_open(my_bool create_if_missing,
-                                        my_bool print_error)
+                                        my_bool print_error,
+                                        my_bool wait_for_lock)
 {
   uchar buffer[CF_MAX_SIZE];
   char name[FN_REFLEN], errmsg_buff[256];
@@ -311,8 +314,9 @@ CONTROL_FILE_ERROR ma_control_file_open(my_bool create_if_missing,
       errmsg= "Can't create file";
       goto err;
     }
-    if (lock_control_file(name))
+    if (lock_control_file(name, wait_for_lock))
     {
+      error= CONTROL_FILE_LOCKED;
       errmsg= lock_failed_errmsg;
       goto err;
     }
@@ -320,7 +324,6 @@ CONTROL_FILE_ERROR ma_control_file_open(my_bool create_if_missing,
   }
 
   /* Otherwise, file exists */
-
   if ((control_file_fd= mysql_file_open(key_file_control, name,
                                         open_flags, MYF(MY_WME))) < 0)
   {
@@ -328,8 +331,10 @@ CONTROL_FILE_ERROR ma_control_file_open(my_bool create_if_missing,
     goto err;
   }
 
-  if (lock_control_file(name)) /* lock it before reading content */
+  /* lock it before reading content */
+  if (lock_control_file(name, wait_for_lock))
   {
+    error= CONTROL_FILE_LOCKED;
     errmsg= lock_failed_errmsg;
     goto err;
   }
@@ -531,7 +536,7 @@ int ma_control_file_write_and_force(LSN last_checkpoint_lsn_arg,
                 "Control file must be from a newer version; zero-ing out %u"
                 " unknown bytes in control file at offset %u", zeroed,
                 cf_changeable_size + cf_create_time_size);
-    ma_message_no_user(ME_JUST_WARNING, msg);
+    ma_message_no_user(ME_WARNING, msg);
   }
   else
   {
@@ -583,8 +588,8 @@ int ma_control_file_end(void)
 
   close_error= mysql_file_close(control_file_fd, MYF(MY_WME));
   /*
-    As mysql_file_close() frees structures even if close() fails, we do the same,
-    i.e. we mark the file as closed in all cases.
+    As mysql_file_close() frees structures even if close() fails, we do the
+    same, i.e. we mark the file as closed in all cases.
   */
   control_file_fd= -1;
   /*
@@ -608,4 +613,124 @@ my_bool ma_control_file_inited(void)
   return (control_file_fd >= 0);
 }
 
+/**
+   Print content of aria_log_control file
+*/
+
+my_bool print_aria_log_control()
+{
+  uchar buffer[CF_MAX_SIZE];
+  char name[FN_REFLEN], uuid_str[MY_UUID_STRING_LENGTH+1];
+  const char *errmsg;
+  uint new_cf_create_time_size, new_cf_changeable_size;
+  my_off_t file_size;
+  ulong logno;
+  ulonglong trid,checkpoint_lsn;
+  int open_flags= O_BINARY | /*O_DIRECT |*/ O_RDWR | O_CLOEXEC;
+  int error= CONTROL_FILE_UNKNOWN_ERROR;
+  uint recovery_fails;
+  File file;
+  DBUG_ENTER("ma_control_file_open");
+
+  if (fn_format(name, CONTROL_FILE_BASE_NAME,
+                maria_data_root, "", MYF(MY_WME)) == NullS)
+    DBUG_RETURN(CONTROL_FILE_UNKNOWN_ERROR);
+
+  if ((file= mysql_file_open(key_file_control, name,
+                             open_flags, MYF(MY_WME))) < 0)
+  {
+    errmsg= "Can't open file";
+    goto err;
+  }
+
+  file_size= mysql_file_seek(file, 0, SEEK_END, MYF(MY_WME));
+  if (file_size == MY_FILEPOS_ERROR)
+  {
+    errmsg= "Can't read size";
+    goto err;
+  }
+  if (file_size < CF_MIN_SIZE)
+  {
+    /*
+      Given that normally we write only a sector and it's atomic, the only
+      possibility for a file to be of too short size is if we crashed at the
+      very first startup, between file creation and file write. Quite unlikely
+      (and can be made even more unlikely by doing this: create a temp file,
+      write it, and then rename it to be the control file).
+      What's more likely is if someone forgot to restore the control file,
+      just did a "touch control" to try to get Maria to start, or if the
+      disk/filesystem has a problem.
+      So let's be rigid.
+    */
+    error= CONTROL_FILE_TOO_SMALL;
+    errmsg= "Size of control file is smaller than expected";
+    goto err;
+  }
+
+  /* Check if control file is unexpectedly big */
+  if (file_size > CF_MAX_SIZE)
+  {
+    error= CONTROL_FILE_TOO_BIG;
+    errmsg= "File size bigger than expected";
+    goto err;
+  }
+
+  if (mysql_file_pread(file, buffer, (size_t)file_size, 0, MYF(MY_FNABP)))
+  {
+    errmsg= "Can't read file";
+    goto err;
+  }
+
+  if (memcmp(buffer + CF_MAGIC_STRING_OFFSET,
+             CF_MAGIC_STRING, CF_MAGIC_STRING_SIZE))
+  {
+    error= CONTROL_FILE_BAD_MAGIC_STRING;
+    errmsg= "Missing valid id at start of file. File is not a valid aria control file";
+    goto err;
+  }
+
+  printf("Aria file version:   %u\n", buffer[CF_VERSION_OFFSET]);
+
+  new_cf_create_time_size= uint2korr(buffer + CF_CREATE_TIME_SIZE_OFFSET);
+  new_cf_changeable_size=  uint2korr(buffer + CF_CHANGEABLE_SIZE_OFFSET);
+
+  if (new_cf_create_time_size < CF_MIN_CREATE_TIME_TOTAL_SIZE ||
+      new_cf_changeable_size <  CF_MIN_CHANGEABLE_TOTAL_SIZE ||
+      new_cf_create_time_size + new_cf_changeable_size != file_size)
+  {
+    error= CONTROL_FILE_INCONSISTENT_INFORMATION;
+    errmsg= "Sizes stored in control file are inconsistent";
+    goto err;
+  }
+  checkpoint_lsn= lsn_korr(buffer + new_cf_create_time_size +
+                           CF_LSN_OFFSET);
+  logno= uint4korr(buffer + new_cf_create_time_size + CF_FILENO_OFFSET);
+  my_uuid2str(buffer + CF_UUID_OFFSET, uuid_str);
+  uuid_str[MY_UUID_STRING_LENGTH]= 0;
+
+  printf("Block size:          %u\n", uint2korr(buffer + CF_BLOCKSIZE_OFFSET));
+  printf("maria_uuid:          %s\n", uuid_str);
+  printf("last_checkpoint_lsn: " LSN_FMT "\n", LSN_IN_PARTS(checkpoint_lsn));
+  printf("last_log_number:     %lu\n", (ulong) logno);
+  if (new_cf_changeable_size >= (CF_MAX_TRID_OFFSET + CF_MAX_TRID_SIZE))
+  {
+    trid= transid_korr(buffer + new_cf_create_time_size + CF_MAX_TRID_OFFSET);
+    printf("trid:                %llu\n", (ulonglong) trid);
+  }
+  if (new_cf_changeable_size >= (CF_RECOV_FAIL_OFFSET + CF_RECOV_FAIL_SIZE))
+  {
+    recovery_fails=
+      (buffer + new_cf_create_time_size + CF_RECOV_FAIL_OFFSET)[0];
+    printf("recovery_failures:   %u\n", recovery_fails);
+  }
+
+  DBUG_RETURN(0);
+
+err:
+  my_printf_error(HA_ERR_INITIALIZATION,
+                  "Got error '%s' when trying to use aria control file "
+                  "'%s'", 0, errmsg, name);
+  DBUG_RETURN(error);
+}
+
 #endif /* EXTRACT_DEFINITIONS */
